// Copyright (c) 2011 Sundog Software, LLC. All rights reserved worldwide.

#ifndef SILVERLINING_SCENIX
#define SILVERLINING_SCENIX

#include "nvsg/Scene.h"
#include "nvsg/Primitive.h"
#include "SilverLining.h"

/** SilverLiningManager handles the integration of SilverLining with a SceniX group node 
(presumably your root node.) */
class SilverLiningManager
{
public:
    SilverLiningManager() {}

    /** Wraps the given group node with a SilverLining sky and clouds. Returns NULL if
    SilverLining failed to initialize, or an Atmosphere object that may then be used to
    configure the atmospheric conditions, time, and location. The SilverLiningManager will
    take responsibility for the memory management of this Atmosphere. 
    
    If you pass in a handle to a directed light, SilverLining will update the light to match
    the simulated conditions each frame. 
    */
    SilverLining::Atmosphere *AttachToRoot(nvsg::GroupSharedPtr rootNode, nvsg::CameraSharedPtr camera,
        nvsg::DirectedLightHandle *light);
};

/// This Drawable-derived class is used to wrap the SilverLining sky box. It is used
/// by the SilverLiningManager and you shouldn't need to use it directly.
class SilverLiningSky : public nvsg::Primitive
{
public:
    SilverLiningSky() : Primitive() {}
    virtual ~SilverLiningSky();

    nvmath::Sphere3f  calculateBoundingSphere () const;

    nvmath::Box3f calculateBoundingBox () const;
};

// Custom draw callback for the SilverLiningSky.
class SilverLiningSkyDraw : public nvsg::DrawCallback
{
public:
    SilverLiningSkyDraw(SilverLining::Atmosphere *pAtmosphere) : atmosphere(pAtmosphere), DrawCallback() {}

    virtual bool draw(const nvsg::Drawable *drawable);

private:
    void extractMatrices(nvsg::CameraSharedPtr camera);
    void updateLight(nvsg::DirectedLightHandle *light);

    SilverLining::Atmosphere *atmosphere;
};

/// This Drawable-derived class is used to wrap the SilverLining clouds and precipitation. It is used
/// by the SilverLiningManager and you shouldn't need to use it directly.
class SilverLiningClouds : public nvsg::Primitive
{
public:
    SilverLiningClouds() : Primitive() { }

    nvmath::Sphere3f  calculateBoundingSphere () const;

    nvmath::Box3f calculateBoundingBox() const;
};

/// Custom draw callback for the SilverLiningClouds.
class SilverLiningCloudsDraw : public nvsg::DrawCallback
{
public:
    SilverLiningCloudsDraw(SilverLining::Atmosphere *pAtmosphere) : atmosphere(pAtmosphere), DrawCallback() {}
    virtual bool draw(const nvsg::Drawable *drawable);

private:
    SilverLining::Atmosphere *atmosphere;
};

CORE_TYPES( SilverLiningSky, nvsg::Primitive );
CORE_TYPES( SilverLiningClouds, nvsg::Primitive );

#endif